#include <Arduino.h>

#include <Automaton.h>
#include "Atm_lien/Atm_stepper.h"

#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>
#include <TeensyMAC.h>
#include <OSCMessage.h>
// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network:
byte mac[] = {
 0x00, 0xAA, 0xBB, 0xCC, 0xDE, 0x02
};

IPAddress ip(192, 168, 1, 204);    //local IP of Arduino/Teensy
//unsigned int localPort = 8888;      // local port to listen on (not needed for multicast)

IPAddress ipMulti(239, 200, 200, 200); // ipMidi Multicast address
unsigned int portMulti = 9977;   // ipMidi Mutlicast port 1


// buffers for receiving and sending data
byte packetBuffer[UDP_TX_PACKET_MAX_SIZE];       // buffer to hold incoming packet
byte sendBuffer1[] = {0x90, 0x14, 0x22};        // MIDI Note On to Multicast address
byte sendBuffer2[] = {0x80, 0x14, 0x00};        // MIDI Note Off to Multicast address

// An EthernetUDP instance to let us send and receive packets over UDP
EthernetUDP Udp;

Atm_stepper stepper;

class Atm_blink : public Machine {

 public:
   Atm_blink( void ) : Machine() {};

   short pin;
   atm_timer_millis timer;

   EthernetUDP* _udpRef ;
   //Atm_blink& onOSC(OSCMessage& msg );

   enum { IDLE, LED_ON, LED_OFF }; // STATES
   enum { EVT_TIMER, EVT_ON, EVT_OFF, ELSE }; // EVENTS
   enum { ENT_ON, ENT_OFF }; // ACTIONS

   Atm_blink & begin( int attached_pin, uint32_t blinkrate, EthernetUDP& udpRef  ) {
     const static state_t state_table[] PROGMEM = {
       /*            ON_ENTER    ON_LOOP  ON_EXIT  EVT_TIMER  EVT_ON  EVT_OFF  ELSE */
       /* IDLE    */  ENT_OFF,        -1,      -1,        -1, LED_ON,      -1,   -1,
       /* LED_ON  */   ENT_ON,        -1,      -1,   LED_OFF,     -1,    IDLE,   -1,
       /* LED_OFF */  ENT_OFF,        -1,      -1,    LED_ON,     -1,    IDLE,   -1,
     };
     Machine::begin( state_table, ELSE );
     pin = attached_pin;
     timer.set( blinkrate );
     pinMode( pin, OUTPUT );
     this->_udpRef =   &udpRef;
     return *this;
   }

   int event( int id ) {
     switch ( id ) {
       case EVT_TIMER :
         return timer.expired( this );
     }
     return 0;
   }

   void action( int id ) {

     switch ( id ) {
       case ENT_ON :
         digitalWrite( pin, HIGH );

         return;

       case ENT_OFF :
         digitalWrite( pin, LOW );
         return;
     }
   }

   Atm_blink& onOSC(OSCMessage& msg ){
     if(msg.fullMatch("/truc")){
       OSCMessage _msg("/blink");
       _msg.add("youpi");
       _udpRef->beginPacket(ipMulti, portMulti);
       _msg.send(*_udpRef); // send the bytes to the SLIP stream
       _udpRef->endPacket(); // mark the end of the OSC Packet
       _msg.empty();
     }
   }
};

Atm_blink led;

void test(OSCMessage &msg, int addrOffset){
msg.getInt(0)?led.trigger( led.EVT_ON ):led.trigger( led.EVT_OFF );
}

void routeTone(OSCMessage &msg, int addrOffset ){
//iterate through all the analog pins
for(byte pin = 0; pin < NUM_DIGITAL_PINS; pin++){
  //match against the pin number strings
  int pinMatched = msg.match("5", addrOffset);
  if(pinMatched){
    unsigned int frequency = 0;
    //if it has an int, then it's an integers frequency in Hz
    if (msg.isInt(0)){
      frequency = msg.getInt(0);
    } //otherwise it's a floating point frequency in Hz
    else if(msg.isFloat(0)){
      frequency = msg.getFloat(0);
    }
    else
      noTone(pin);
    if(frequency>0)
    {
       if(msg.isInt(1))
         tone(pin, frequency, msg.getInt(1));
       else
         tone(pin, frequency);
    }
  }
}
}

void setup() {
 SPI.setSCK(27);
 Ethernet.init(10);
 teensyMAC(mac);
 Ethernet.begin(mac, ip);              // start the Ethernet and UDP:
//  Udp.beginMulti(ipMulti, portMulti);   // for modified Arduino library
 Udp.beginMulticast(ipMulti, portMulti); // for modified Teensy Ethernet library
 Serial.begin(115200); // higher Baud rate for faster refresh, CHANGE IN SERIAL MONITOR

 led.begin( 14, 200, Udp);        // Setup a blink machine on pin 4
 led.trigger( led.EVT_ON );  // Turn it on

  Serial.begin( 9600 );
  delay(1000);
  stepper.trace( Serial );
  Serial.println("Started");
  stepper.begin(2,1);
  stepper.cycle(1000);
  Serial.println("Started");
  stepper.gotoStep(16000);
  Serial.println("Started");
  // stepper.trigger(stepper.EVT_GOTO);
  // pinMode(00, OUTPUT);
  // digitalWrite(0, HIGH);
}

void loop() {
  automaton.run();
 OSCMessage msgIn;
 int size;

 if( (size = Udp.parsePacket())>0)
 {
   while(size--)
     msgIn.fill(Udp.read());

    if(!msgIn.hasError())
      msgIn.route("/blink", test);
      led.onOSC(msgIn);
 }

 // OSCMessage msg("/analog/0");
 // msg.add((int32_t)analogRead(0));
 //
 // Udp.beginPacket(ipMulti, portMulti);
 //   msg.send(Udp); // send the bytes to the SLIP stream
 // Udp.endPacket(); // mark the end of the OSC Packet
 // msg.empty(); // free space occupied by message

 // led.cycle();

 //delay(10);
 //UDPsend();   // sending of packets; do not use with UDPreceive because it's including delay()!!
 //UDPreceive();

}
